import { InjectionToken, ɵɵdefineInjectable, ɵɵinject, Injectable, Optional, Inject, PLATFORM_ID, Directive, Injector, EventEmitter, ElementRef, Input, Output, HostListener, INJECTOR, NgModule, APP_INITIALIZER } from '@angular/core';
import { isPlatformBrowser, DOCUMENT, isPlatformServer, CommonModule } from '@angular/common';
import { MatSelect } from '@angular/material/select';
import { fromEvent, of } from 'rxjs';
import { map, debounceTime, distinctUntilChanged } from 'rxjs/operators';
import { MediaMatcher } from '@angular/cdk/layout';
import { Router, ActivatedRoute } from '@angular/router';
import { NgModel, FormControlName } from '@angular/forms';

class NgxMatSelectConfigs {
    constructor() {
        this.emptyLabel = 'No results found';
        this.searchBoxPlaceholder = 'Search ...';
        // selectedItemsLabel: string = "Selected Item(s)";
        this.maximumResultForShow = 30;
        this.valueMember = 'id';
        this.displayMember = 'name';
        this.maxWidthForMobileView = 599.9;
        this.useMobileView = true;
        this.mobileViewType = 'FullScreen';
    }
}
const NGX_MAT_SELECT_CONFIGS = new InjectionToken('NGX_MAT_SELECT_CONFIGS');

class NgxMatSelectService {
    constructor(_configs) {
        this.configs = Object.assign(new NgxMatSelectConfigs(), _configs);
    }
}
NgxMatSelectService.ɵprov = ɵɵdefineInjectable({ factory: function NgxMatSelectService_Factory() { return new NgxMatSelectService(ɵɵinject(NGX_MAT_SELECT_CONFIGS, 8)); }, token: NgxMatSelectService, providedIn: "root" });
NgxMatSelectService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
NgxMatSelectService.ctorParameters = () => [
    { type: NgxMatSelectConfigs, decorators: [{ type: Optional }, { type: Inject, args: [NGX_MAT_SELECT_CONFIGS,] }] }
];

class NgxMatSelectMediaTracker {
    constructor(injector) {
        this._mobileQueryListener = () => {
            if (this.onChangeMediaTracker)
                this.onChangeMediaTracker();
        };
        this.configs = injector.get(NgxMatSelectService).configs;
        this.media = injector.get(MediaMatcher);
        this.platformId = injector.get(PLATFORM_ID);
        if (isPlatformBrowser(this.platformId)) {
            this._mobileQueryListener();
        }
    }
    ngOnDestroy() {
        if (isPlatformBrowser(this.platformId)) {
            this.mobileQuery.removeListener(this._mobileQueryListenerFunc);
        }
    }
    ngOnInit() {
        if (isPlatformBrowser(this.platformId)) {
            this.mobileQuery = this.media.matchMedia('(max-width: ' + this.configs.maxWidthForMobileView + 'px)');
            this._mobileQueryListenerFunc = this._mobileQueryListener.bind(this);
            this.mobileQuery.addListener(this._mobileQueryListenerFunc);
            this.onChangeMediaTracker();
        }
    }
}
NgxMatSelectMediaTracker.decorators = [
    { type: Directive }
];
NgxMatSelectMediaTracker.ctorParameters = () => [
    { type: Injector }
];

class NgxMatSelectDirective extends NgxMatSelectMediaTracker {
    constructor(injector) {
        super(injector);
        this.injector = injector;
        this._lastSourceLen = 0;
        this.lazyLoad = false;
        this.source = [];
        this.filteredSourceChange = new EventEmitter();
        this.filteredSource = [];
        this.isStringArray = false;
        this.fragment = 'search';
        this.hasSearchBox = true;
        this._isFirstLoading = false;
        this._pageNumber = 1;
        this._hasMoreRow = true;
        this.searchValue = '';
        this._htmlScrollTop = 0;
        this._htmlScrollLeft = 0;
        this._bodyScrollTop = 0;
        this._bodyScrollLeft = 0;
        this.loadingContainer = `<div class="ngx-mat-select-loading-container"><div class="loadingio-spinner-ellipsis-ao6bq46b8nt"><div class="ldio-it58uqzfg5">
  <div></div><div></div><div></div><div></div><div></div>
</div></div></div>`;
        this.loadingScrollContainer = `<div class="loadingio-spinner-rolling-2vwh2v284qu"><div class="ldio-o6nze7by54l">
                                                                                <div></div>
                                                                                       </div></div>`;
        this.onChangeMediaTracker = () => {
            this._changeMedia();
        };
        this._scrollProcess = false;
        this._preventKeyCodeList = [37, 38, 39, 40, 16];
        this.deviceType = () => {
            const ua = navigator.userAgent;
            if (/(tablet|ipad|playbook|silk)|(android(?!.*mobi))/i.test(ua)) {
                return 'tablet';
            }
            else if (/Mobile|Android|iP(hone|od)|IEMobile|BlackBerry|Kindle|Silk-Accelerated|(hpw|web)OS|Opera M(obi|ini)/.test(ua)) {
                return 'mobile';
            }
            return 'desktop';
        };
        this.matSelect = injector.get(MatSelect);
        this._openPanelSub = this.matSelect._openedStream.subscribe(this._afterOpen.bind(this));
        this._currentEl = injector.get(ElementRef).nativeElement;
        this.router = injector.get(Router, null);
        this.route = injector.get(ActivatedRoute, null);
        setTimeout(() => {
            this._handleValueChanges();
        });
    }
    setloading(check) {
        if (this._listBoxEl) {
            const icon = this._listBoxEl.querySelector('.ngx-mat-select-search-box-icon');
            const loading = this._listBoxEl.querySelector('.ngx-mat-select-loading-container');
            if (check) {
                icon.style.display = 'none';
                loading.style.display = 'flex';
            }
            else {
                icon.style.display = 'block';
                loading.style.display = 'none';
            }
            if (this._mainLoadingElement) {
                const el = this._getListMainContainerElement();
                if (check) {
                    el.scrollTo({
                        top: 0,
                    });
                    this._mainLoadingElement.style.display = 'block';
                    el.style.overflow = 'hidden';
                }
                else {
                    setTimeout(() => {
                        el.style.overflow = 'auto';
                        this._mainLoadingElement.style.display = 'none';
                    }, 100);
                }
            }
        }
    }
    ngOnInit() {
        super.ngOnInit();
        this._populateConfigs();
        this._closePanelSub = this.matSelect._closedStream.subscribe(this._afterClose.bind(this));
    }
    ngAfterViewInit() {
        setTimeout(() => {
            this._bindAfterClick();
        }, 50);
        if (this.useMobileView) {
            this._changeMedia();
        }
        this.matSelect.compareWith = (o1, o2) => {
            if (o1 && o2) {
                if (typeof o1 === 'object') {
                    return o1[this.valueMember] === o2[this.valueMember];
                }
                else {
                    return o1 === o2;
                }
            }
        };
        if (this.inFirstLoadCallSearcher &&
            this.searcher &&
            typeof this.searcher === 'function') {
            this._isFirstLoading = true;
            this.searchValue = this.inFirstLoadSearcherValue;
            try {
                this._onSearch()
                    .then((res) => {
                    this.source = res;
                    this._isFirstLoading = false;
                    this.searchValue = '';
                })
                    .catch(() => {
                    this._isFirstLoading = false;
                    this.searchValue = '';
                });
            }
            catch (_a) {
                this._isFirstLoading = false;
                this.searchValue = '';
            }
        }
    }
    _checkIsFirstLoading() {
        setTimeout(() => {
            if (this._isFirstLoading) {
                this._checkIsFirstLoading();
            }
            else {
                setTimeout(() => {
                    this.setloading(false);
                    this._changeSearchBoxEnableStatus(true);
                }, 250);
            }
        }, 150);
    }
    ngOnDestroy() {
        super.ngOnDestroy();
        if (this._openPanelSub)
            this._openPanelSub.unsubscribe();
        if (this._closePanelSub) {
            this._closePanelSub.unsubscribe();
        }
        if (this._selectionChangeSub) {
            this._selectionChangeSub.unsubscribe();
        }
        if (this._inputSubscription) {
            this._inputSubscription.unsubscribe();
        }
        if (this._valueChangeSub) {
            this._valueChangeSub.unsubscribe();
        }
        if (this._valueChangeSub) {
            this._valueChangeSub.unsubscribe();
        }
        if (this.matSelectEl) {
            this.matSelectEl.removeEventListener('click', this.clickMatSelectFunc);
        }
    }
    ngOnChanges(changes) {
        if (changes.source && changes.source.currentValue) {
            setTimeout(() => {
                if (changes.source.firstChange) {
                    this._populateConfigs();
                }
                this._init();
            });
        }
    }
    onPopState(event) {
        var _a;
        (_a = this.matSelect) === null || _a === void 0 ? void 0 : _a.close();
    }
    onClick(event) {
        if (!this.matSelect.panelOpen && this.filteredSource.length == 0) {
            this._addNullKeyToOption();
            setTimeout(() => {
                this.matSelect.open();
                this._cleanNullKeyToOption();
            });
        }
    }
    _populateConfigs() {
        var _a, _b;
        if (this.emptyLabel === undefined) {
            this.emptyLabel = this.configs.emptyLabel;
        }
        if (this.searchBoxPlaceholder === undefined) {
            this.searchBoxPlaceholder = this.configs.searchBoxPlaceholder;
        }
        if (this.maximumResultForShow === undefined) {
            this.maximumResultForShow = this.configs.maximumResultForShow;
        }
        if (this.useMobileView === undefined || this.useMobileView == null) {
            this.useMobileView = this.configs.useMobileView;
        }
        if (!this.mobileViewType) {
            this.mobileViewType = this.configs.mobileViewType;
        }
        if (!this.valueMember) {
            this.valueMember = this.configs.valueMember;
        }
        if (!this.displayMember) {
            this.displayMember = this.configs.displayMember;
        }
        if (!this.noMoreResultLabel) {
            this.noMoreResultLabel = this.configs.noMoreResultLabel;
            if (!this.noMoreResultLabel) {
                this.noMoreResultLabel = 'No more results found';
            }
        }
        if (this.useInfiniteScroll === undefined ||
            this.useInfiniteScroll === null) {
            this.useInfiniteScroll = this.configs.useInfiniteScroll;
        }
        if (this.inFirstLoadCallSearcher === undefined ||
            this.inFirstLoadCallSearcher === null) {
            this.inFirstLoadCallSearcher = this.configs.inFirstLoadCallSearcher;
        }
        if (this.inFirstLoadSearcherValue === undefined ||
            this.inFirstLoadSearcherValue == null) {
            this.inFirstLoadSearcherValue = (_a = this.configs.inFirstLoadSearcherValue) !== null && _a !== void 0 ? _a : '';
        }
        (_b = this._clickableEl) === null || _b === void 0 ? void 0 : _b.removeEventListener('click', this._afterClickFunc);
    }
    _bindAfterClick() {
        this._afterClickFunc = this.onClick.bind(this);
        const parentEl = this._currentEl.parentElement.parentElement.parentElement.parentElement;
        if (parentEl.tagName == 'MAT-FORM-FIELD') {
            this._clickableEl = parentEl;
            parentEl.addEventListener('click', this._afterClickFunc);
        }
        else {
            this._currentEl.addEventListener('click', this._afterClickFunc);
            this._clickableEl = this._currentEl;
        }
    }
    _handleValueChanges() {
        const ngModelChanges = this.injector.get(NgModel, null);
        if (ngModelChanges) {
            this._valueChangeSub = ngModelChanges.valueChanges.subscribe((val) => {
                if (val)
                    this._valueChangeDetected();
            });
        }
        else {
            const formControlValueChange = this.injector.get(FormControlName, null);
            if (formControlValueChange) {
                this._valueChangeSub = formControlValueChange.valueChanges.subscribe((val) => {
                    if (val)
                        this._valueChangeDetected();
                });
            }
            else {
                this._valueChangeSub = this.matSelect.valueChange.subscribe((val) => {
                    if (val)
                        this._valueChangeDetected();
                });
            }
        }
    }
    _valueChangeDetected() {
        if (!this.matSelect.panelOpen) {
            this._fromSearch = false;
            if (this.searchValue) {
                this.searchValue = '';
            }
            this._syncValueAndOptions();
            setTimeout(() => {
                this._syncSourceAndOptions(this.filteredSource);
            });
        }
    }
    _addLoadingElementForScrollInf() {
        const container = document.createElement('div');
        container.classList.add('ngx-mat-select-scroll-loading');
        container.innerHTML = this.loadingScrollContainer;
        this._scrollInfLoadingEl = container;
        this._listBoxEl.parentElement.appendChild(container);
    }
    changeScrollLoadingState(show) {
        if (this._scrollInfLoadingEl) {
            if (show) {
                this._changeSearchBoxEnableStatus(false);
                this._scrollInfLoadingEl.style.display = 'block';
            }
            else {
                this._scrollInfLoadingEl.style.display = 'none';
                this._changeSearchBoxEnableStatus(true);
            }
        }
    }
    _addMainLoading() {
        const el = document.createElement('div');
        el.classList.add('ngx-mat-select-main-loading');
        el.innerHTML = this.loadingScrollContainer;
        this._mainLoadingElement = el;
        this._listBoxEl.appendChild(el);
    }
    scrollInfChange(event) {
        const el = event.target;
        if (el) {
            const scrollTop = el.scrollTop;
            const scrollHeight = el.scrollHeight;
            const height = el.clientHeight;
            if (scrollTop + height + 20 >= scrollHeight &&
                !this._scrollProcess &&
                this._hasMoreRow) {
                this._scrollProcess = true;
                const checkSourceLen = this.source.length || this._lastSourceLen;
                if (checkSourceLen < this.maximumResultForShow) {
                    this._hasMoreRow = false;
                    this._scrollProcess = false;
                }
                else {
                    this.changeScrollLoadingState(true);
                    setTimeout(() => {
                        this._listBoxEl.scrollTo({
                            top: this._listBoxEl.scrollHeight,
                        });
                        setTimeout(() => {
                            this._pageNumber += 1;
                            this._onSearch(true).then((items) => {
                                this._lastSourceLen = items.length;
                                const len = items.length;
                                this._hasMoreRow = len >= this.maximumResultForShow;
                                this.changeScrollLoadingState(false);
                                items.forEach((z) => {
                                    z['ngxPage'] = this._pageNumber;
                                    z.ngxHide = false;
                                });
                                let pagedSrc = [
                                    ...this.filteredSource.filter((w) => !w['ngxPage'] || w['ngxPage'] != this._pageNumber),
                                    ...items,
                                ];
                                this._syncSourceAndOptions(pagedSrc.filter((w) => !w.ngxHide), this._pageNumber * this.maximumResultForShow, pagedSrc);
                                if (!this._hasMoreRow) {
                                    this._changeNoMoreResultState(true);
                                    this._scrollToNextItem();
                                }
                                else {
                                    this._scrollToNextItem();
                                }
                                this._scrollProcess = false;
                            });
                        }, 350);
                    }, 0);
                }
            }
        }
    }
    _addScrollInf() {
        this.scrollInfFunc = this.scrollInfChange.bind(this);
        const el = this._getListMainContainerElement();
        el === null || el === void 0 ? void 0 : el.addEventListener('scroll', this.scrollInfFunc, false);
        this._addLoadingElementForScrollInf();
    }
    _changeNoMoreResultState(show) {
        if (this._noMoreResultEl) {
            if (!show) {
                this._noMoreResultEl.style.display = 'none';
            }
            else {
                this._noMoreResultEl.style.display = 'block';
            }
        }
    }
    _scrollToNextItem() {
        var _a, _b, _c;
        const el = this._getListMainContainerElement();
        const optionHeight = (_c = (_b = (_a = this.matSelect) === null || _a === void 0 ? void 0 : _a.options) === null || _b === void 0 ? void 0 : _b.first._getHostElement().clientHeight) !== null && _c !== void 0 ? _c : 34;
        this._scrollInList(el.scrollTop + optionHeight);
    }
    _scrollToEndOfList() {
        const el = this._getListMainContainerElement();
        this._scrollInList(el.scrollHeight);
    }
    _scrollInList(top) {
        const el = this._getListMainContainerElement();
        if (el) {
            setTimeout(() => {
                el.scrollTo({
                    top: top,
                    behavior: 'smooth',
                });
            });
        }
    }
    _getListMainContainerElement() {
        let el;
        if (this.mobileQuery.matches && this.useMobileView) {
            el = this._listBoxEl.parentElement;
        }
        else {
            el = this._listBoxEl;
        }
        return el;
    }
    _addNoMoreResultElement() {
        if (this._listBoxEl) {
            const container = document.createElement('div');
            container.classList.add('ngx-mat-select-scroll-no-more');
            container.innerText = this.noMoreResultLabel;
            this._noMoreResultEl = container;
            this._listBoxEl.appendChild(container);
        }
    }
    _removeScrollInfEvent() {
        const el = this._getListMainContainerElement();
        if (el) {
            el.removeEventListener('scroll', this.scrollInfFunc);
        }
    }
    _changeSearchBoxEnableStatus(enable) {
        if (this._searchBoxInputEl) {
            if (enable) {
                this._searchBoxInputEl.removeAttribute('disabled');
            }
            else {
                this._searchBoxInputEl.setAttribute('disabled', 'disabled');
            }
        }
    }
    _cleanNullKeyToOption() {
        const index = this.filteredSource.findIndex((z) => z.keyNull == 'null');
        if (index >= 0) {
            this.filteredSource.splice(index, 1);
            if (this.filteredSource.length == 0) {
                this._syncSourceAndOptions(this.filteredSource);
            }
            else {
                this.filteredSourceChange.emit(this.filteredSource);
            }
        }
    }
    _addNullKeyToOption() {
        if (this.matSelect.options.length == 0) {
            this.filteredSource.push({ keyNull: 'null' });
            this.filteredSourceChange.emit(this.filteredSource);
        }
    }
    _moveCursorToEnd(el) {
        if (typeof el.selectionStart == 'number') {
            el.selectionStart = el.selectionEnd = el.value.length;
        }
        else if (typeof el.createTextRange != 'undefined') {
            el.focus();
            let range = el.createTextRange();
            range.collapse(false);
            range.select();
        }
    }
    _handleBackDropAfterClose() {
        const backdropEl = document.querySelector('.cdk-overlay-container.bottomToTop .cdk-overlay-backdrop');
        if (backdropEl) {
            backdropEl.style.display = 'none';
            backdropEl.parentElement.classList.remove('bottomToTop');
        }
    }
    _handleOverflowAfterClose() {
        var _a;
        if (this.useMobileView) {
            if (this.mobileQuery.matches) {
                document.documentElement.style.overflow = '';
                setTimeout(() => {
                    document.documentElement.scrollTo(this._htmlScrollLeft, this._htmlScrollTop);
                }, 0);
                if (this.fragment) {
                    (_a = this.router) === null || _a === void 0 ? void 0 : _a.navigate([], { relativeTo: this.route, fragment: null });
                }
            }
        }
    }
    _handleSelectionAfterClose() {
        this._addNullKeyToOption();
    }
    _addSearchablePanelClass(parent) {
        if (this._listBoxEl) {
            if (!parent.classList.contains('ngx-mat-select-panel')) {
                parent.classList.add('ngx-mat-select-panel');
            }
        }
    }
    _createHeaderElement() {
        this.timeOut = null;
        const searchElement = document.createElement('div');
        searchElement.innerHTML = `
    <input type='text'

     placeholder="${this.searchBoxPlaceholder}"
     class="ngx-mat-select-search-box-input" value="${this.searchValue}" >
    <i class="ngx-mat-select-search-box-icon"></i>
    ${this.loadingContainer}
    `;
        searchElement.addEventListener('keydown', (ev) => {
            if ((ev === null || ev === void 0 ? void 0 : ev.keyCode) === 32) {
                ev.stopImmediatePropagation();
            }
        });
        const inputContainerEl = document.createElement('div');
        inputContainerEl.classList.add('ngx-mat-select-search-box-wrap');
        inputContainerEl.appendChild(searchElement);
        searchElement.classList.add('ngx-mat-select-search-box-container');
        searchElement.classList.add('abs');
        if (!this.hasSearchBox) {
            if (this.mobileQuery.matches && this.useMobileView) {
                searchElement.style.display = 'none';
            }
            else {
                inputContainerEl.style.display = 'none';
            }
        }
        return inputContainerEl;
    }
    _afterOpen() {
        if (!this.mobileQuery.matches || !this.useMobileView) {
            this._removeOpacity();
        }
        this._listBoxEl = this.matSelect.panel.nativeElement;
        setTimeout(() => {
            if (this.useInfiniteScroll) {
                this._addScrollInf();
                this._addNoMoreResultElement();
            }
            this._addMainLoading();
            if (this._isFirstLoading) {
                this.setloading(true);
                this._changeSearchBoxEnableStatus(false);
                this._checkIsFirstLoading();
            }
        }, 10);
        if (this._listBoxEl) {
            if (this.hasSearchBox &&
                !this.mobileQuery.matches &&
                this.useMobileView) {
                this._listBoxEl.style.height = this._panelMaxHeight + 'px';
            }
            const inputContainerEl = this._createHeaderElement();
            this._listBoxEl.prepend(inputContainerEl);
            this._appendNotFoundHtmlTag();
            this._cleanNullKeyToOption();
            this._handleNotFoundLabel(this.filteredSource);
            if (this.useMobileView) {
                this._afterOpenForMobile(inputContainerEl);
            }
            setTimeout(() => {
                if (this.hasSearchBox) {
                    const inputEl = inputContainerEl.querySelector('input');
                    this._searchBoxInputEl = inputEl;
                    if (!this.mobileQuery.matches || !this.useMobileView) {
                        inputEl.focus();
                        this._moveCursorToEnd(inputEl);
                    }
                    this._handleObserverForInputValue(inputEl);
                }
            });
        }
    }
    _afterClose() {
        this._handleBackDropAfterClose();
        this._handleOverflowAfterClose();
        this._handleSelectionAfterClose();
        this._changeMedia();
        if (this.useInfiniteScroll) {
            this._removeScrollInfEvent();
            this._hasMoreRow = true;
        }
    }
    _changePaging() {
        if (this._pageNumber > 1) {
            this._pageNumber = 1;
            this._syncSourceAndOptions(this.filteredSource.filter((w) => !w.ngxHide), this.maximumResultForShow, this.filteredSource);
        }
    }
    _handleObserverForInputValue(inputEl) {
        const terms$ = fromEvent(inputEl, 'keyup').pipe(map((event) => {
            if (this._checkKeyCode(event)) {
                this.setloading(true);
                return event.target['value'];
            }
            else {
                return event.target['value'];
            }
        }), debounceTime(this.debounceTime()), distinctUntilChanged());
        if (terms$) {
            this._inputSubscription = terms$ === null || terms$ === void 0 ? void 0 : terms$.subscribe((content) => {
                if (content !== undefined && content !== null) {
                    this.searchValue = content;
                    this._onSearch();
                }
            });
        }
    }
    _allowedKeyCode(keyCode) {
        return !this._preventKeyCodeList.includes(keyCode);
    }
    debounceTime() {
        return this.lazyLoad ? 400 : 250;
    }
    _checkKeyCode(event) {
        if (this.deviceType() !== 'desktop') {
            return true;
        }
        if (event && event.keyCode !== null && event.keyCode !== undefined) {
            return this._allowedKeyCode(event.keyCode);
        }
        return true;
    }
    _init() {
        if (this.hasSearchBox) {
            this._syncValueAndOptions();
            setTimeout(() => {
                this._syncSourceAndOptions(this.filteredSource);
            }, 50);
        }
        else {
            this.filteredSource = this.source;
        }
    }
    _syncValueAndOptions(mainSrc = null, maxCount = this.maximumResultForShow) {
        let src = this._setMaxCount(mainSrc || this.source, maxCount);
        this.value = this.matSelect.value;
        if (this.value) {
            const val = this.value;
            if (Array.isArray(val)) {
                if (val.length > 0) {
                    if (src.length > 0) {
                        val.forEach((v) => {
                            const item = src.find((w) => this._getExp(w, v));
                            if (!item) {
                                if (v && typeof v === 'object') {
                                    src.unshift(v);
                                    v['ngxHide'] = true;
                                }
                                else {
                                    const nItem = this.source.find((w) => this._getExp(w, v));
                                    if (nItem) {
                                        src.unshift(nItem);
                                        nItem['ngxHide'] = true;
                                    }
                                }
                            }
                            else {
                                item['ngxHide'] = false;
                            }
                        });
                    }
                    else {
                        if (this.source.length == 0) {
                            this.source = val;
                        }
                        else {
                            val.forEach((item) => (item['ngxHide'] = true));
                        }
                        src = val;
                    }
                }
            }
            else {
                if (src.length > 0) {
                    const item = src.find((w) => this._getExp(w, val));
                    if (!item) {
                        if (val && typeof val === 'object') {
                            val['ngxHide'] = true;
                            src.unshift(val);
                        }
                        else {
                            const nItem = this.source.find((w) => this._getExp(w, val));
                            if (nItem) {
                                nItem['ngxHide'] = true;
                                src.unshift(nItem);
                            }
                        }
                    }
                    else {
                        item['ngxHide'] = false;
                    }
                }
                else if (val && typeof val === 'object') {
                    if (this.source.length == 0) {
                        this.source = [val];
                    }
                    else {
                        val['ngxHide'] = true;
                    }
                    src = [val];
                }
            }
        }
        this.filteredSource = src;
        this.filteredSourceChange.emit(this.filteredSource);
    }
    _syncSourceAndOptions(filteredSource, maxCount = this.maximumResultForShow, mainSrc = this.source) {
        filteredSource = this._setMaxCount(filteredSource, maxCount);
        filteredSource.forEach((item) => (item['ngxSelected'] = 0));
        this.matSelect.options.forEach((op) => (op['_element'].nativeElement.style.display = ''));
        let selectedOptionO = this.matSelect.selected;
        let selectedOptions = [];
        if (selectedOptionO) {
            if (!Array.isArray(selectedOptionO)) {
                selectedOptions = [selectedOptionO];
            }
            else {
                selectedOptions = selectedOptionO;
            }
            let unshiftedlist = [];
            selectedOptions.forEach((option) => {
                let item = filteredSource.find((z) => this._getExpForOption(z, option.value));
                if (!item) {
                    if (option.value && typeof option.value === 'object') {
                        item = option.value;
                    }
                    else {
                        item = mainSrc.find((z) => this._getExpForOption(z, option.value));
                    }
                    if (item) {
                        item['ngxSelected'] = 1;
                        item['ngxHide'] = true;
                        unshiftedlist.push(item);
                    }
                }
                else if (item['ngxHide']) {
                    if (this._fromSearch) {
                        item['ngxHide'] = false;
                    }
                    else {
                        item['ngxHide'] = true;
                    }
                }
            });
            filteredSource.unshift(...unshiftedlist);
        }
        const showNotFoundLabel = this._handleNotFoundLabel(filteredSource);
        if (showNotFoundLabel) {
            this._toggleParentClass(this.matSelect.panel, 'remove-options', '.cdk-overlay-pane');
        }
        else {
            this._toggleParentClass(this.matSelect.panel, 'hide-options', '.cdk-overlay-pane');
        }
        this.filteredSource = filteredSource;
        this.filteredSourceChange.emit(this.filteredSource);
        setTimeout(() => {
            this._hideOptions();
            setTimeout(() => {
                this._removeParentClass(this.matSelect.panel, 'hide-options', '.cdk-overlay-pane');
                this._removeParentClass(this.matSelect.panel, 'remove-options', '.cdk-overlay-pane');
            }, 0);
        });
    }
    _hideOptions() {
        this.filteredSource
            .filter((z) => !!z['ngxHide'])
            .forEach((hide) => {
            let ops;
            if (!this.matSelect.multiple) {
                ops = [this.matSelect.selected];
            }
            else {
                ops = this.matSelect.selected;
            }
            const option = ops.find((z) => this._getExpForOption(hide, z.value));
            if (option) {
                const opEl = option['_element'].nativeElement;
                opEl.style.display = 'none';
            }
        });
    }
    _getPagedResult(src) {
        let result = [];
        const startIndex = this.maximumResultForShow * (this._pageNumber - 1);
        let endIndex = this.maximumResultForShow * this._pageNumber;
        if (endIndex > src.length) {
            endIndex = src.length;
        }
        const len = src.length;
        if (startIndex <= len) {
            result = src.slice(startIndex, endIndex);
        }
        return result;
    }
    refreshData() {
        return new Promise((resolve, reject) => {
            if (!this.matSelect.panelOpen) {
                this._isFirstLoading = true;
            }
            else {
                this.setloading(true);
                this._changeSearchBoxEnableStatus(false);
            }
            try {
                this._onSearch()
                    .then((res) => {
                    this._isFirstLoading = false;
                    this._changeSearchBoxEnableStatus(true);
                    this.source = res;
                    resolve(res);
                })
                    .catch(() => {
                    this._isFirstLoading = false;
                    this._changeSearchBoxEnableStatus(true);
                    reject();
                });
            }
            catch (_a) {
                this._isFirstLoading = false;
                this._changeSearchBoxEnableStatus(true);
                reject();
            }
        });
    }
    _onSearch(changePage = false) {
        this._fromSearch = true;
        if (!changePage) {
            if (this._listBoxEl) {
                this._listBoxEl.scrollTo({
                    top: 0,
                });
            }
            this._pageNumber = 1;
            this._hasMoreRow = true;
            this._changeNoMoreResultState(false);
        }
        return new Promise((resolve, reject) => {
            try {
                this._addSearchablePanelClass(this._listBoxEl);
                if (!this.lazyLoad) {
                    let filteredSource = [];
                    if (this.isStringArray) {
                        filteredSource = this.source.filter((x) => x === null || x === void 0 ? void 0 : x.toString().includes(this.searchValue));
                    }
                    else {
                        filteredSource = this.source.filter((x) => { var _a; return (_a = x[this.displayMember]) === null || _a === void 0 ? void 0 : _a.toString().includes(this.searchValue); });
                    }
                    if (!changePage) {
                        this._syncSourceAndOptions(filteredSource);
                        this.setloading(false);
                    }
                    else {
                        filteredSource = this._getPagedResult(filteredSource);
                    }
                    resolve(filteredSource);
                }
                else {
                    if (this.searcher && typeof this.searcher === 'function') {
                        this.searcher(this.searchValue, this._pageNumber, this.maximumResultForShow)
                            .toPromise()
                            .then((res) => {
                            if (!changePage) {
                                this._syncSourceAndOptions(res);
                                this.setloading(false);
                            }
                            resolve(res);
                        })
                            .catch(() => {
                            this.setloading(false);
                            resolve(0);
                        });
                    }
                    else {
                        this.setloading(false);
                        resolve(0);
                    }
                }
            }
            catch (_a) {
                resolve(0);
            }
        });
    }
    _appendNotFoundHtmlTag() {
        const el = document.createElement('div');
        el.classList.add('ngx-mat-select-not-found-container');
        el.innerHTML = this.emptyLabel;
        this._listBoxEl.appendChild(el);
    }
    _handleNotFoundLabel(filteredSource) {
        const show = filteredSource && filteredSource.length > 0
            ? filteredSource.filter((z) => !z['ngxHide']).length == 0
            : true;
        if (this._listBoxEl) {
            const el = this._listBoxEl.querySelector('.ngx-mat-select-not-found-container');
            if (el) {
                if (show) {
                    el.style.display = 'flex';
                }
                else {
                    el.style.display = 'none';
                }
            }
        }
        return show;
    }
    _setMaxCount(filteredSource, maxCount = this.maximumResultForShow) {
        if (maxCount) {
            filteredSource = filteredSource.slice(0, this._getMaxSize(filteredSource, maxCount));
        }
        return filteredSource;
    }
    _getExp(left, right) {
        if (right[this.valueMember]) {
            return left[this.valueMember] == right[this.valueMember];
        }
        else {
            return left[this.valueMember] == right;
        }
    }
    _getExpForOption(left, right) {
        if (left[this.valueMember]) {
            if (right[this.valueMember]) {
                return left[this.valueMember] == right[this.valueMember];
            }
            else {
                return left[this.valueMember] == right;
            }
        }
        else {
            return left == right;
        }
    }
    _getMaxSize(arr, maximumCount) {
        if (!arr)
            return 0;
        const length = arr.length;
        return length > maximumCount ? maximumCount : length;
    }
    _changeMedia() {
        var _a;
        let classes = (_a = this.matSelect.panelClass) === null || _a === void 0 ? void 0 : _a.toString().split(' ');
        if (!classes) {
            classes = [];
        }
        if (!classes.includes('ngx-mat-select-opacity')) {
            classes.push('ngx-mat-select-opacity');
        }
        if (this.mobileQuery.matches && this.useMobileView) {
            const hasMyClass = classes.includes('ngx-mat-select-mobile');
            if (this.matSelect) {
                if (!hasMyClass) {
                    classes.push('ngx-mat-select-mobile');
                }
            }
        }
        else {
            classes = classes.filter((z) => z !== 'ngx-mat-select-mobile');
        }
        if (classes)
            this.matSelect.panelClass = classes.join(' ');
    }
    _addTitleForMobileView(spacer) {
        if (spacer && !this.hasSearchBox) {
            const titleEl = document.createElement('div');
            titleEl.classList.add('ngx-mat-select-mobile-title');
            if (this.title)
                titleEl.innerHTML = this.title;
            if (document.documentElement.dir === 'rtl') {
                titleEl.style.right = '48px';
            }
            else {
                titleEl.style.left = '48px';
            }
            spacer.style.display = 'block';
            spacer.prepend(titleEl);
        }
    }
    _removeOpacity() {
        setTimeout(() => {
            this._listBoxEl.classList.remove('ngx-mat-select-opacity');
        });
    }
    _afterOpenForMobile(spacer) {
        var _a;
        this._htmlScrollTop = document.documentElement.scrollTop;
        this._htmlScrollLeft = document.documentElement.scrollLeft;
        const panel = this.matSelect.panel;
        const panelEl = panel.nativeElement;
        const parentEl = panelEl.parentElement;
        if (this.mobileQuery.matches) {
            document.documentElement.style.overflow = 'hidden';
            panelEl.classList.remove('mat-select-panel');
            if (this.hasSearchBox || this.title) {
                const listEl = document.querySelector('.ngx-mat-select-mobile');
                if (listEl) {
                    listEl.style.paddingTop = '50px';
                }
            }
            this._addTitleForMobileView(spacer);
            setTimeout(() => {
                this._removeOpacity();
            }, 80);
            this._toggleParentClass(panel, 'ngx-mat-select-mobile-wrapper', '.cdk-overlay-pane');
            if (this.mobileViewType === 'BottomSheet') {
                this._toggleParentClass(panel, 'bottomToTop', '.cdk-overlay-container');
                this._toggleParentClass(panel, 'ngx-mat-select-mobile-open-bottomToTop', '.cdk-overlay-pane');
            }
            else {
                this._toggleParentClass(panel, 'ngx-mat-select-mobile-open-fullScreen', '.cdk-overlay-pane');
            }
            parentEl.style.overflowY = 'auto';
            parentEl.style.overflowX = 'hidden';
            const firstSelected = panelEl.querySelector('.mat-selected');
            setTimeout(() => {
                firstSelected === null || firstSelected === void 0 ? void 0 : firstSelected.scrollIntoView({ block: 'center', behavior: 'smooth' });
            }, 500);
            if (this.hasSearchBox || this.title || true) {
                const iconAppend = spacer;
                const inputSearch = spacer.querySelector('.mat-select-search-input');
                if (inputSearch)
                    inputSearch.style.padding = '0';
                const btn = document.createElement('i');
                if (document.documentElement.dir === 'rtl') {
                    btn.classList.add('arrow-right');
                }
                else {
                    btn.classList.add('arrow-left');
                }
                btn.onclick = (ev) => {
                    ev.stopImmediatePropagation();
                    this.matSelect.close();
                };
                iconAppend.style.display = 'flex';
                iconAppend.style.alignItems = 'center';
                iconAppend.style.width = '100%';
                iconAppend === null || iconAppend === void 0 ? void 0 : iconAppend.prepend(btn);
            }
            if (this.fragment) {
                (_a = this.router) === null || _a === void 0 ? void 0 : _a.navigate([], {
                    relativeTo: this.route,
                    fragment: this.fragment,
                });
            }
            spacer.style.position = 'fixed';
            setTimeout(() => {
                spacer.style.position = '';
            }, 150);
        }
        else {
            if (!panelEl.classList.contains('mat-select-panel')) {
                panel.nativeElement.classList.add('mat-select-panel');
            }
        }
    }
    _toggleParentClass(el, className, targetSelect) {
        var _a;
        if ((_a = el === null || el === void 0 ? void 0 : el.nativeElement) === null || _a === void 0 ? void 0 : _a.parentElement) {
            const parentEl = el.nativeElement.parentElement.closest(targetSelect);
            if (parentEl) {
                parentEl.classList.toggle(className, true);
            }
        }
    }
    _removeParentClass(el, className, targetSelect) {
        var _a;
        if ((_a = el === null || el === void 0 ? void 0 : el.nativeElement) === null || _a === void 0 ? void 0 : _a.parentElement) {
            const parentEl = el.nativeElement.parentElement.closest(targetSelect);
            if (parentEl) {
                parentEl.classList.remove(className);
            }
        }
    }
}
NgxMatSelectDirective.decorators = [
    { type: Directive, args: [{
                selector: 'mat-select[ngxMatSelect]',
                exportAs: 'ngxMatSelect',
            },] }
];
NgxMatSelectDirective.ctorParameters = () => [
    { type: Injector }
];
NgxMatSelectDirective.propDecorators = {
    lazyLoad: [{ type: Input }],
    source: [{ type: Input }],
    filteredSourceChange: [{ type: Output }],
    filteredSource: [{ type: Input }],
    searcher: [{ type: Input }],
    isStringArray: [{ type: Input }],
    fragment: [{ type: Input }],
    hasSearchBox: [{ type: Input }],
    title: [{ type: Input }],
    emptyLabel: [{ type: Input }],
    searchBoxPlaceholder: [{ type: Input }],
    maximumResultForShow: [{ type: Input }],
    useMobileView: [{ type: Input }],
    mobileViewType: [{ type: Input }],
    valueMember: [{ type: Input }],
    displayMember: [{ type: Input }],
    value: [{ type: Input }],
    noMoreResultLabel: [{ type: Input }],
    useInfiniteScroll: [{ type: Input }],
    inFirstLoadCallSearcher: [{ type: Input }],
    inFirstLoadSearcherValue: [{ type: Input }],
    onPopState: [{ type: HostListener, args: ['window:popstate', ['$event'],] }]
};

class NgxMatSelectInitializer {
    constructor(injector) {
        this._mobileQueryListener = () => {
            if (this.mobileQuery) {
                this._handleClasses(this.mobileQuery.matches);
            }
        };
        this.media = injector.get(MediaMatcher);
        this.platformId = injector.get(PLATFORM_ID);
        this.configs = injector.get(NgxMatSelectService).configs;
        this.document = injector.get(DOCUMENT);
    }
    _handleClasses(matches) {
        if (matches) {
            if (!document.documentElement.classList.contains('ngx-mat-mobile')) {
                document.documentElement.classList.add('ngx-mat-mobile');
            }
        }
        else {
            document.documentElement.classList.remove('ngx-mat-mobile');
        }
    }
    init() {
        return () => {
            if (isPlatformServer(this.platformId)) {
                return of(true).toPromise();
            }
            return new Promise((resolve, reject) => {
                if (window.screen.width <= this.configs.maxWidthForMobileView) {
                    this._mobileQueryListener();
                }
                this.mobileQuery = this.media.matchMedia('(max-width: '
                    + this.configs.maxWidthForMobileView
                    + 'px)');
                this._mobileQueryListenerFunc = this._mobileQueryListener.bind(this);
                this.mobileQuery.addListener(this._mobileQueryListenerFunc);
                this._handleClasses(this.document.documentElement.clientWidth <= this.configs.maxWidthForMobileView);
                {
                    this._handleClasses(this.mobileQuery.matches);
                }
                resolve(true);
            });
        };
    }
}
NgxMatSelectInitializer.ɵprov = ɵɵdefineInjectable({ factory: function NgxMatSelectInitializer_Factory() { return new NgxMatSelectInitializer(ɵɵinject(INJECTOR)); }, token: NgxMatSelectInitializer, providedIn: "root" });
NgxMatSelectInitializer.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
NgxMatSelectInitializer.ctorParameters = () => [
    { type: Injector }
];

function nInit(ngxMatSelectInitializer) {
    return ngxMatSelectInitializer.init();
}
class NgxMatSelectModule {
    static forRoot(configs) {
        if (!configs) {
            configs = new NgxMatSelectConfigs();
        }
        else {
            configs = Object.assign(new NgxMatSelectConfigs(), configs);
        }
        return {
            ngModule: NgxMatSelectModule,
            providers: [
                { provide: NGX_MAT_SELECT_CONFIGS, useValue: configs },
                NgxMatSelectService,
            ]
        };
    }
}
NgxMatSelectModule.decorators = [
    { type: NgModule, args: [{
                declarations: [NgxMatSelectDirective],
                imports: [
                    CommonModule,
                ],
                providers: [
                    NgxMatSelectInitializer,
                    {
                        provide: APP_INITIALIZER, deps: [NgxMatSelectInitializer],
                        useFactory: nInit,
                        multi: true
                    }
                ],
                exports: [
                    NgxMatSelectDirective,
                ],
            },] }
];

/*
 * Public API Surface of ngx-mat-select
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NGX_MAT_SELECT_CONFIGS, NgxMatSelectConfigs, NgxMatSelectDirective, NgxMatSelectInitializer, NgxMatSelectMediaTracker, NgxMatSelectModule, NgxMatSelectService, nInit };
//# sourceMappingURL=ngx-mat-select.js.map
